/*this file modified by Tom Pratum, July 2007*/
/*derived from files originally in SpinCore spinapi dated 2-28-2007*/

#include <string.h>
#include <stdio.h>
#include <stdlib.h>
#include <stdarg.h>
#include <time.h>
#include <errno.h>
#include "util.h"

// setting this to 1 causes the debug() function to print out lots of debugging
// info
static int do_debug = 0;

extern char *spinerr;


/**
 * Enable debug log. When enabled, spinapi will generate a file called log.txt,
 * which contains some debugging information. This file is probably not very useful
 * for the end-user, but if you are encountering difficulty, it will help us to turn
 * debugging on and email us the log along with your support question.
 * \param debug Set to 1 to turn on debugging outputs, or 0 to turn off.
 */
void pb_set_debug(int debug) {
    do_debug = debug;    
}

/**
 * Return a string which is of the form:<br>
 * a: b
 *
 */
char *my_strcat(char *a, char *b)
{
	char *string;

	string = malloc	(sizeof(char)*(strlen(a) + strlen(b) + 5));

	strcpy(string,a);
	strcat(string,": ");
	strcat(string,b);
	
	return string;
}

char *my_sprintf (char *format, ...)
{
	va_list ap;
	char *string;
	
	string = (char *)malloc(1024 * sizeof(char));
	
	va_start(ap, format);
	vsprintf(string, format, ap);
	va_end (ap);


	return string;	
}

/**
 This is a different debugging function - if debug is defined above then debug prints, othewise it does nothing; the only difference with respect to what was here before is that this function prints to the terminal rather than a file */

void debug (char *format, ...)
{
	va_list ap;


	// Only output text if debugging is enabled
	if (!do_debug) {
		return;	
	}

	va_start (ap, format);	
	vprintf(format, ap);	
	va_end(ap);
}

/*second level debugging*/
void debug2 (char *format, ...)
{
	va_list ap;


	// Only output text if debugging is enabled
	if (!do_debug || do_debug ==1) {
		return;	
	}

	va_start (ap, format);	
	vprintf(format, ap);	
	va_end(ap);
}


/** Here is the old debug function
 * Print debugging information to log.txt. log.txt is created the first
 * time this function is called. The arguments to this function work in the
 * same was as printf()
 */
/*void debug (char *format, ...)
{
	va_list ap;
	time_t t;

	static FILE *f;

	// Only output text if debugging is enabled
	if (!do_debug) {
		return;	
	}

	// If the log file has not yet been opened, open it
	if (!f) {
		f = fopen ("log.txt", "w");	

		// If the open failed for some reason, print errors to stdout instead
		if (!f) {
			printf ("DEBUG WARNING: Couldnt open log.txt (%s). Using stdout instead\n", strerror(errno));
			f = stdout;
		}

		// Print some information as a header
		t = time(0);
		fprintf (f, "Logfile created: %s\n", asctime(localtime(&t)));
		fprintf (f, "Spinapi version: %s\n", pb_get_version());
		//fprintf (f, "Firmware ID: 0x%x\n", pb_get_firmware_id());
		fprintf (f, "================================================\n");
	}
	
	va_start (ap, format);	
	vfprintf(f, format, ap);	
	va_end(ap);

    fflush(f);

}*/
