/*this file modified by Tom Pratum, July 2007*/
/* spinapi.h
 * API definitions for the spinapi library.
 *
 * $Date: 2006/12/13 20:46:37 $
 * 
 * this was modified slightly by tkp 070907 for OSX
 *
 * To get the latest version of this code, or to contact us for support, please
 * visit http://www.spincore.com
 */

/* Copyright (c) 2006 SpinCore Technologies, Inc.
 *
 * This software is provided 'as-is', without any express or implied warranty. 
 * In no event will the authors be held liable for any damages arising from the 
 * use of this software.
 *
 * Permission is granted to anyone to use this software for any purpose, 
 * including commercial applications, and to alter it and redistribute it
 * freely, subject to the following restrictions:
 *
 * 1. The origin of this software must not be misrepresented; you must not
 * claim that you wrote the original software. If you use this software in a
 * product, an acknowledgment in the product documentation would be appreciated
 * but is not required.
 * 2. Altered source versions must be plainly marked as such, and must not be
 * misrepresented as being the original software.
 * 3. This notice may not be removed or altered from any source distribution.
 */

/**
 *\file spinapi.h
 *\brief Main header file which defines all publicly accessible functions
 *
 */


#ifndef _SPINAPI_H
#define _SPINAPI_H

//Defines for start_programming
#define PULSE_PROGRAM  0
#define FREQ_REGS      1

#define PHASE_REGS     2
#define TX_PHASE_REGS  2
#define PHASE_REGS_1   2

#define RX_PHASE_REGS  3
#define PHASE_REGS_0   3

// These are names used by RadioProcessor
#define COS_PHASE_REGS 51
#define SIN_PHASE_REGS 50

// For specifying which device in pb_dds_load
#define DEVICE_SHAPE 0x099000
#define DEVICE_DDS   0x099001

//Defines for enabling analog output
#define ANALOG_ON 1
#define ANALOG_OFF 0
#define TX_ANALOG_ON 1
#define TX_ANALOG_OFF 0
#define RX_ANALOG_ON 1
#define RX_ANALOG_OFF 0

//Defines for different pb_inst instruction types
#define CONTINUE 0
#define STOP 1
#define LOOP 2
#define END_LOOP 3
#define JSR 4
#define RTS 5
#define BRANCH 6
#define LONG_DELAY 7
#define WAIT 8

//Defines for using different units of time
#define ns 1.0
#define us 1000.0
#define ms 1000000.0
// This causes problems with some versions of stdio.h
//#define s 1000000000.0

//Defines for using different units of frequency
#define MHz 1.0
#define kHz .001
#define Hz .000001

#define PARAM_ERROR -99

//Variables for max number of registers (Currently the same across models)
#define MAX_PHASE_REGS 16
#define MAX_FREQ_REGS 16

/// \brief Overflow counter structure
///
/// This structure holds the values of the various onboard overflow counters. These counters
/// stop counting once they reach 65535.
typedef struct {
    /// Number of overflows that occur when sampling data at the ADC
    int adc; 
    /// Number of overflows that occur after the CIC filter
    int cic;
    /// Number of overflows that occur after the FIR filter
    int fir;
    /// Number of overflows that occur during the averaging process
    int average;
} PB_OVERFLOW_STRUCT;


int pb_count_boards ();
int pb_select_board (int board_num);
int pb_init();
void pb_set_clock(double clock_freq);
int pb_close();
int pb_start_programming(int device);
int pb_set_freq(double freq);
int pb_set_phase(double phase);
int pb_inst_tworf(int freq, int tx_phase, int tx_output_enable, int rx_phase, int rx_output_enable, int flags, int inst, int inst_data, double length);
int pb_inst_onerf(int freq, int phase, int rf_output_enable, int flags, int inst, int inst_data, double length);
int pb_inst_pbonly(int flags, int inst, int inst_data, double length);
int pb_inst_direct(int flags, int inst, int inst_data_direct, int length);
int pb_stop_programming();
int pb_start();
int pb_stop();
int pb_outp(unsigned int address, char data);
char pb_inp(unsigned int address);
int pb_outw(unsigned int address, unsigned int data);
unsigned int pb_inw(unsigned int address);
void pb_set_ISA_address(int address);
double pb_get_rounded_value();
int pb_read_status();
int pb_read_fullStat(int address);
char* pb_get_version();
char* pb_get_error();
int pb_get_firmware_id();
void pb_sleep_ms(int milliseconds);
void pb_set_debug(int debug);
void pb_bypass_FF_fix(int option);

// RadioProcessor related functions
int pb_set_defaults();
int pb_inst_radio(int freq, int cos_phase, int sin_phase, int tx_phase, int tx_enable, int phase_reset, int trigger_scan, int flags, int inst, int inst_data, double length);
int pb_inst_radio_shape(int freq, int cos_phase, int sin_phase, int tx_phase, int tx_enable, int phase_reset, int trigger_scan, int use_shape, int amp, int flags, int inst, int inst_data, double length);
int pb_set_num_points(int num_points);
int pb_set_scan_segments(int num_segments);
int pb_scan_count(int reset);
int pb_zero_ram();
int pb_overflow(int reset, PB_OVERFLOW_STRUCT *of);
int pb_get_data(int num_points, int *real_data, int *imag_data);
int pb_get_data_direct(int num_points, short *data);
int pb_write_ascii (char * fname, int num_points, float SW, int *real_data, int *imag_data);
int pb_write_ascii_verbose (char * fname, int num_points, float SW, float SF, int *real_data, int *imag_data);
int pb_write_jcamp(char * fname, int num_points, float SW, float SF, int *real_data, int *imag_data);
int pb_write_felix(char * fnameout, int num_points, float SaW, float SF, int *real_data, int *imag_data);
int pb_setup_filters(double spectral_width, int scan_repetitions, int cmd);
int pb_setup_cic (int dec_amount, int shift_amount, int m, int stages);
int pb_load_coef_file (int *coef, char *fname, int num_coefs);
int pb_setup_fir (int num_taps, int *coef, int shift_amount, int dec_amount);
int pb_set_radio_control(unsigned int control);
int pb_unset_radio_control (unsigned int control);
int pb_set_radio_hw(int adc_control, int dac_control);

int pb_dds_load(float *data, int device);
int pb_set_amp(float amp, int addr);

// Legacy function names
void set_clock(double clock_freq);
int start_programming(int device);
int stop_programming();
int start_pb();
int stop_pb();
int set_freq(double freq);
int set_phase(double phase);
int read_status();


//RadioProcessor control word defines
#define TRIGGER             0x0001
#define PCI_READ            0x0002
#define BYPASS_AVERAGE      0x0004
#define NARROW_BW           0x0008
#define FORCE_AVG			0x0010
#define BNC0_CLK            0x0020
#define DO_ZERO             0x0040
#define BYPASS_CIC          0x0080
#define BYPASS_FIR          0x0100
#define BYPASS_MULT         0x0200
#define SELECT_AUX_DDS      0x0400
#define DDS_DIRECT          0x0800
#define SELECT_INTERNAL_DDS 0x1000
#define DAC_FEEDTHROUGH     0x2000
#define OVERFLOW_RESET      0x4000
#define RAM_DIRECT          0x8000

#ifdef PB24
#define pb_inst pb_inst_pbonly
#endif

#ifdef PBDDS
#define pb_inst pb_inst_tworf
#define PHASE_RESET 0x200
#endif

#ifdef PBESR
#define pb_inst pb_inst_pbonly
#endif

#ifdef PBESRPRO
#define pb_inst pb_inst_pbonly
#endif

#define ALL_FLAGS_ON	0x1FFFFF
#define ONE_PERIOD		0x200000
#define TWO_PERIOD		0x400000
#define THREE_PERIOD	0x600000
#define FOUR_PERIOD		0x800000
#define FIVE_PERIOD		0xA00000
#define SIX_PERIOD      0xC00000
#define ON				0xE00000

// maximum number of boards that can be supported
/* this was defined below, but my code only accomodates 8 boards
   this can easily be changed in driver_usb_osx.c */
/* #define MAX_NUM_BOARDS 32 */
#define MAX_NUM_BOARDS 8

#endif /* #ifdef _SPINAPI_H */
